@@ -25,12 +25,15 @@
 
 #include "collectioncommand.h"
 #include "../collection.h"
+#include "../collections/bibtexcollection.h"
 #include "../document.h"
 #include "../controller.h"
 #include "../tellico_debug.h"
 
 #include <KLocalizedString>
 
+#include <algorithm>
+
 using Tellico::Command::CollectionCommand;
 
 CollectionCommand::CollectionCommand(Mode mode_, Tellico::Data::CollPtr origColl_, Tellico::Data::CollPtr newColl_)
@@ -43,7 +46,10 @@ CollectionCommand::CollectionCommand(Mode mode_, Tellico::Data::CollPtr origColl
 #ifndef NDEBUG
 // just some sanity checking
   if(!m_origColl || !m_newColl) {
-    myDebug() << "null collection pointer";
+    myDebug() << "CommandTest: null collection pointer";
+  }
+  if(m_origColl != Data::Document::self()->collection()) {
+    myWarning() << "CollectionCommand: original collection is different than the current document";
   }
 #endif
   switch(m_mode) {
@@ -80,12 +86,27 @@ void CollectionCommand::redo() {
   switch(m_mode) {
     case Append:
       copyFields();
-      Data::Document::self()->appendCollection(m_newColl);
+      copyMacros();
+      {
+        auto existingEntries = m_origColl->entryIdList();
+        Data::Document::self()->appendCollection(m_newColl);
+        auto allEntries = m_origColl->entryIdList();
+
+        // keep track of which entries were added by the append operation
+        // by taking difference of the entry id lists
+        m_addedEntries.clear();
+        std::sort(existingEntries.begin(), existingEntries.end());
+        std::sort(allEntries.begin(), allEntries.end());
+        std::set_difference(allEntries.begin(), allEntries.end(),
+                            existingEntries.begin(), existingEntries.end(),
+                            std::back_inserter(m_addedEntries));
+      }
       Controller::self()->slotCollectionModified(m_origColl);
       break;
 
     case Merge:
       copyFields();
+      copyMacros();
       m_mergePair = Data::Document::self()->mergeCollection(m_newColl);
       Controller::self()->slotCollectionModified(m_origColl);
       break;
@@ -108,12 +129,14 @@ void CollectionCommand::undo() {
 
   switch(m_mode) {
     case Append:
-      Data::Document::self()->unAppendCollection(m_newColl, m_origFields);
+      unCopyMacros();
+      Data::Document::self()->unAppendCollection(m_origFields, m_addedEntries);
       Controller::self()->slotCollectionModified(m_origColl);
       break;
 
     case Merge:
-      Data::Document::self()->unMergeCollection(m_newColl, m_origFields, m_mergePair);
+      unCopyMacros();
+      Data::Document::self()->unMergeCollection(m_origFields, m_mergePair);
       Controller::self()->slotCollectionModified(m_origColl);
       break;
 
@@ -133,3 +156,47 @@ void CollectionCommand::copyFields() {
     m_origFields.append(Data::FieldPtr(new Data::Field(*field)));
   }
 }
+
+void CollectionCommand::copyMacros() {
+  // only applies to bibliographies
+  if(m_origColl->type() != Data::Collection::Bibtex ||
+     m_newColl->type() != Data::Collection::Bibtex) {
+    return;
+  }
+  m_addedMacros.clear();
+  // iterate over all macros in the new collection, check if they exist in the orig, add them if not
+  // do not over write them
+  // TODO: what to do if they clash?
+  auto origColl = static_cast<Data::BibtexCollection*>(m_origColl.data());
+  const QMap<QString, QString> origMacros = origColl->macroList();
+  const QMap<QString, QString> newMacros = static_cast<Data::BibtexCollection*>(m_newColl.data())->macroList();
+
+  auto i = newMacros.constBegin();
+  while(i != newMacros.constEnd()) {
+    if(!origMacros.contains(i.key())) {
+      origColl->addMacro(i.key(), i.value());
+      m_addedMacros.insert(i.key(), i.value());
+    }
+    ++i;
+  }
+
+  m_origPreamble = origColl->preamble();
+  if(m_origPreamble.isEmpty()) {
+    origColl->setPreamble(static_cast<Data::BibtexCollection*>(m_newColl.data())->preamble());
+  }
+}
+
+void CollectionCommand::unCopyMacros() {
+  // only applies to bibliographies
+  if(m_origColl->type() != Data::Collection::Bibtex) {
+    return;
+  }
+  auto origColl = static_cast<Data::BibtexCollection*>(m_origColl.data());
+  // remove the macros added by the append/merge
+  auto i = m_addedMacros.constBegin();
+  while(i != m_addedMacros.constEnd()) {
+    origColl->removeMacro(i.key());
+    ++i;
+  }
+  origColl->setPreamble(m_origPreamble);
+}
